# -*- coding: utf-8 -*-
"""
Created on Wed May 15 13:10:01 2024

@author: ikhan1
"""

import pandas as pd
from OCC.Core.BRepBuilderAPI import BRepBuilderAPI_MakeEdge, BRepBuilderAPI_MakeWire
from OCC.Core.BRepOffsetAPI import BRepOffsetAPI_ThruSections
from OCC.Core.GeomAPI import GeomAPI_Interpolate
from OCC.Core.TColgp import TColgp_HArray1OfPnt
from OCC.Core.gp import gp_Pnt
from OCC.Display.SimpleGui import init_display
from OCC.Extend.DataExchange import write_step_file
from OCC.Core.BRepCheck import BRepCheck_Analyzer
from OCC.Core.BRepBuilderAPI import BRepBuilderAPI_MakeVertex
from OCC.Core.TopExp import TopExp_Explorer
from OCC.Core.TopAbs import TopAbs_VERTEX
from OCC.Core.BRep import BRep_Tool

def read_airfoil_data(file_path):
    df = pd.read_excel(file_path)
    airfoils = []
    for i in range(0, len(df), 199):
        points = df.iloc[i:i+199]
        if points.iloc[0].equals(points.iloc[-1]):
            points = points.iloc[:-1]  # Ensure no duplicate end points
        airfoil_points = [(x, y, z) for x, y, z in zip(points.iloc[:, 0], points.iloc[:, 1], points.iloc[:, 2])]
        
        # Reverse the order of points for airfoils from the 8th onwards
        if i >= 7 * 199:
            airfoil_points = airfoil_points[::-1]
        
        airfoils.append(airfoil_points)
    return airfoils

def create_wire_from_points(points):
    point_array = TColgp_HArray1OfPnt(1, len(points))
    for i, point in enumerate(points, start=1):
        point_array.SetValue(i, gp_Pnt(*point))
    
    # Use GeomAPI_Interpolate to generate a closed BSpline curve from points
    interpolator = GeomAPI_Interpolate(point_array, True, 1e-6)  # Set closed to True to ensure closure
    interpolator.Perform()
    if not interpolator.IsDone():
        raise RuntimeError("Interpolation failed")
    
    spline = interpolator.Curve()
    edge = BRepBuilderAPI_MakeEdge(spline).Edge()
    wire_builder = BRepBuilderAPI_MakeWire()
    wire_builder.Add(edge)
    if not wire_builder.IsDone():
        raise RuntimeError("Wire construction failed")
    
    wire = wire_builder.Wire()
    
    analyzer = BRepCheck_Analyzer(wire)
    if not analyzer.IsValid():
        raise RuntimeError("Invalid wire detected")
    
    return wire

def interpolate_airfoils(airfoil1, airfoil2, factor=0.5):
    interpolated = [(p1[0] * (1 - factor) + p2[0] * factor,
                     p1[1] * (1 - factor) + p2[1] * factor,
                     p1[2] * (1 - factor) + p2[2] * factor) for p1, p2 in zip(airfoil1, airfoil2)]
    return interpolated

def loft_wires(wires):
    loft = BRepOffsetAPI_ThruSections(True, True)  # Create a solid loft
    for wire in wires:
        loft.AddWire(wire)
    loft.CheckCompatibility(False)  # Ensure compatibility is not checked to avoid modifications
    loft.Build()
    if not loft.IsDone():
        raise RuntimeError("Lofting failed to create a valid shape")
    return loft.Shape()

def visualize_points(points, display):
    for point in points:
        vertex = BRepBuilderAPI_MakeVertex(gp_Pnt(*point)).Vertex()
        display.DisplayShape(vertex, update=True)

def visualize_wire(wire, display):
    exp = TopExp_Explorer(wire, TopAbs_VERTEX)
    while exp.More():
        vertex = exp.Current()
        display.DisplayShape(vertex, update=True)
        exp.Next()

# Commented out debugging functions
# def debug_wire_info(wire, label):
#     print(f"Debugging wire: {label}")
#     exp = TopExp_Explorer(wire, TopAbs_VERTEX)
#     while exp.More():
#         vertex = exp.Current()
#         pnt = BRep_Tool.Pnt(vertex)
#         print(f"Vertex: {pnt.X()}, {pnt.Y()}, {pnt.Z()}")
#         exp.Next()

def are_wires_compatible(wire1, wire2):
    # Implement detailed compatibility checks between two wires
    return True

def validate_wire_sections(wires):
    for i in range(len(wires) - 1):
        wire1 = wires[i]
        wire2 = wires[i + 1]
        if not are_wires_compatible(wire1, wire2):
            print(f"Wires at index {i} and {i + 1} are not compatible")
            return False
    return True

def main():
    display, start_display, add_menu, add_function_to_menu = init_display()
    file_path = r'C:\Users\ikhan1\Downloads\original_blade.xlsx'
    airfoils = read_airfoil_data(file_path)
    wires = []
    
    for i, airfoil in enumerate(airfoils):
        visualize_points(airfoil, display)
        wire = create_wire_from_points(airfoil)
        visualize_wire(wire, display)
        # debug_wire_info(wire, f"Airfoil {i}")
        wires.append(wire)
        if i < len(airfoils) - 1:
            if i == 6:  # Special handling for the transition
                for factor in [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9]:
                    intermediate_airfoil = interpolate_airfoils(airfoils[i], airfoils[i + 1], factor)
                    intermediate_wire = create_wire_from_points(intermediate_airfoil)
                    visualize_wire(intermediate_wire, display)
                    # debug_wire_info(intermediate_wire, f"Intermediate {i} - factor {factor}")
                    wires.append(intermediate_wire)
            else:
                intermediate_airfoil = interpolate_airfoils(airfoils[i], airfoils[i + 1])
                intermediate_wire = create_wire_from_points(intermediate_airfoil)
                visualize_wire(intermediate_wire, display)
                # debug_wire_info(intermediate_wire, f"Intermediate {i}")
                wires.append(intermediate_wire)
    
    try:
        for wire in wires:
            display.DisplayShape(wire, update=True)
        
        if validate_wire_sections(wires):
            wing = loft_wires(wires)
            write_step_file(wing, "wing_model.step")
            display.DisplayShape(wing, update=True)
        else:
            print("Wire sections are not compatible for lofting.")
    except RuntimeError as e:
        print(f"Error during lofting: {e}")
    
    start_display()

if __name__ == "__main__":
    main()
